@@ -9,6 +9,7 @@ import re
 import shutil
 import stat
 import sys
+from typing import Any, Callable
 from urllib.parse import urlparse
 
 import botocore
@@ -16,10 +17,30 @@ import durationpy
 
 LOG = logging.getLogger(__name__)
 
+
+class UnserializableJsonError(TypeError):
+    """Exception class for JSON encoding errors"""
+
+    pass
+
+
 ##
 # Settings / Packaging
 ##
 
+# mimetypes starting with entries defined here are considered as TEXT when BINARTY_SUPPORT is True.
+# - Additional TEXT mimetypes may be defined with the 'ADDITIONAL_TEXT_MIMETYPES' setting.
+DEFAULT_TEXT_MIMETYPES = (
+    "text/",
+    "application/json",  # RFC 4627
+    "application/javascript",  # RFC 4329
+    "application/ecmascript",  # RFC 4329
+    "application/xml",  # RFC 3023
+    "application/xml-external-parsed-entity",  # RFC 3023
+    "application/xml-dtd",  # RFC 3023
+    "image/svg+xml",  # RFC 3023
+)
+
 
 def copytree(src, dst, metadata=True, symlinks=False, ignore=None):
     """
@@ -244,6 +265,10 @@ def get_event_source(event_source, lambda_arn, target_function, boto_session, dr
             super().__init__(context, config)
             self._lambda = kappa.awsclient.create_client("lambda", context.session)
 
+        @property
+        def batch_window(self):
+            return self._config.get("batch_window", 1 if self.batch_size > 10 else 0)
+
         def _get_uuid(self, function):
             uuid = None
             response = self._lambda.call(
@@ -263,6 +288,7 @@ def get_event_source(event_source, lambda_arn, target_function, boto_session, dr
                     FunctionName=function.name,
                     EventSourceArn=self.arn,
                     BatchSize=self.batch_size,
+                    MaximumBatchingWindowInSeconds=self.batch_window,
                     Enabled=self.enabled,
                 )
                 LOG.debug(response)
@@ -301,6 +327,7 @@ def get_event_source(event_source, lambda_arn, target_function, boto_session, dr
                     response = self._lambda.call(
                         "update_event_source_mapping",
                         BatchSize=self.batch_size,
+                        MaximumBatchingWindowInSeconds=self.batch_window,
                         Enabled=self.enabled,
                         FunctionName=function.arn,
                     )
@@ -592,3 +619,85 @@ def merge_headers(event):
     for h in multi_headers.keys():
         multi_headers[h] = ", ".join(multi_headers[h])
     return multi_headers
+
+
+class ApacheNCSAFormatters:
+    """
+    NCSA extended/combined Log Format:
+    "%h %l %u %t \"%r\" %>s %b \"%{Referer}i\" \"%{User-agent}i\""
+    %h: Remote hostname.
+    %l: Remote logname
+    %u: Remote user if the request was authenticated. May be bogus if return status (%s) is 401 (unauthorized).
+    %t: Time the request was received, in the format [18/Sep/2011:19:18:28 -0400].
+        The last number indicates the timezone offset from GMT
+    %r: First line of request.
+    %>s: Final Status
+    %b: Size of response in bytes, excluding HTTP headers.
+        In CLF format, i.e. a '-' rather than a 0 when no bytes are sent.
+    %{Referer}i:The contents of Referer: header line(s) in the request sent to the server.
+    %{User-agent}i: The contents of User-agent: header line(s) in the request sent to the server.
+
+    Refer to:
+    https://httpd.apache.org/docs/current/en/mod/mod_log_config.html
+    """
+
+    @staticmethod
+    def format_log(status_code: int, environ: dict, content_length: int, **kwargs) -> str:
+        ip_header = kwargs.get("ip_header", None)
+        if ip_header:
+            host = environ.get(ip_header, "")
+        else:
+            host = environ.get("REMOTE_ADDR", "")
+
+        logname = "-"
+        user = "-"
+        now = datetime.datetime.now(datetime.timezone.utc)
+        display_datetime = now.strftime("%d/%b/%Y:%H:%M:%S %z")
+        method = environ.get("REQUEST_METHOD", "")
+        path_info = environ.get("PATH_INFO", "")
+        query_string = ""
+        raw_query_string = environ.get("QUERY_STRING", "")
+        if raw_query_string:
+            query_string = f"?{raw_query_string}"
+        server_protocol = environ.get("SERVER_PROTOCOL", "")
+        request = f"{method} {path_info}{query_string} {server_protocol}"
+        referer = environ.get("HTTP_REFERER", "")
+        agent = environ.get("HTTP_USER_AGENT", "")
+        log_entry = (
+            f'{host} {logname} {user} [{display_datetime}] "{request}" {status_code} {content_length} "{referer}" "{agent}"'
+        )
+        return log_entry
+
+    @staticmethod
+    def format_log_with_response_time(*args, **kwargs) -> str:
+        """
+        Expect that kwargs includes response time in microseconds, 'rt_us'.
+        Mimics Apache-like access HTTP log where the response time data is enabled
+
+        NCSA extended/combined Log Format:
+            "%h %l %u %t \"%r\" %>s %b \"%{Referer}i\" \"%{User-agent}i\" %T/%D"
+
+        %T: The time taken to serve the request, in seconds.
+        %D: The time taken to serve the request, in microseconds.
+        """
+        response_time_microseconds = kwargs.get("rt_us", None)
+        log_entry = ApacheNCSAFormatters.format_log(*args, **kwargs)
+        if response_time_microseconds:
+            response_time_seconds = int(response_time_microseconds / 1_000_000)
+            log_entry = f"{log_entry} {response_time_seconds}/{response_time_microseconds}"
+        return log_entry
+
+
+def ApacheNCSAFormatter(with_response_time: bool = True) -> Callable:
+    """A factory that returns the wanted formatter"""
+    if with_response_time:
+        return ApacheNCSAFormatters.format_log_with_response_time
+    else:
+        return ApacheNCSAFormatters.format_log
+
+
+def validate_json_serializable(*args: Any, **kwargs: Any) -> None:
+    try:
+        json.dumps((args, kwargs))
+    except (TypeError, OverflowError):
+        raise UnserializableJsonError("Arguments to an asynchronous.task must be JSON serializable!")
