@@ -1,13 +1,12 @@
 import base64
 import logging
 import sys
+from io import BytesIO
 from urllib.parse import urlencode
 
-import six
-from requestlogger import ApacheFormatter
 from werkzeug import urls
 
-from .utilities import merge_headers, titlecase_keys
+from .utilities import ApacheNCSAFormatter, merge_headers, titlecase_keys
 
 BINARY_METHODS = ["POST", "PUT", "PATCH", "DELETE", "CONNECT", "OPTIONS"]
 
@@ -25,26 +24,26 @@ def create_wsgi_request(
     Given some event_info via API Gateway,
     create and return a valid WSGI request environ.
     """
-    method = event_info["httpMethod"]
+    method = event_info.get("httpMethod", None)
     headers = merge_headers(event_info) or {}  # Allow for the AGW console 'Test' button to work (Pull #735)
 
-    """
-        API Gateway and ALB both started allowing for multi-value querystring
-        params in Nov. 2018. If there aren't multi-value params present, then
-        it acts identically to 'queryStringParameters', so we can use it as a
-        drop-in replacement.
+    # API Gateway and ALB both started allowing for multi-value querystring
+    # params in Nov. 2018. If there aren't multi-value params present, then
+    # it acts identically to 'queryStringParameters', so we can use it as a
+    # drop-in replacement.
+    #
+    # The one caveat here is that ALB will only include _one_ of
+    # queryStringParameters _or_ multiValueQueryStringParameters, which means
+    # we have to check for the existence of one and then fall back to the
+    # other.
 
-        The one caveat here is that ALB will only include _one_ of
-        queryStringParameters _or_ multiValueQueryStringParameters, which means
-        we have to check for the existence of one and then fall back to the
-        other.
-        """
     if "multiValueQueryStringParameters" in event_info:
         query = event_info["multiValueQueryStringParameters"]
         query_string = urlencode(query, doseq=True) if query else ""
     else:
         query = event_info.get("queryStringParameters", {})
         query_string = urlencode(query) if query else ""
+    query_string = urls.url_unquote(query_string)
 
     if context_header_mappings:
         for key, value in context_header_mappings.items():
@@ -59,13 +58,6 @@ def create_wsgi_request(
             if header_val is not None:
                 headers[key] = header_val
 
-    # Extract remote user from context if Authorizer is enabled
-    remote_user = None
-    if event_info["requestContext"].get("authorizer"):
-        remote_user = event_info["requestContext"]["authorizer"].get("principalId")
-    elif event_info["requestContext"].get("identity"):
-        remote_user = event_info["requestContext"]["identity"].get("userArn")
-
     # Related:  https://github.com/Miserlou/Zappa/issues/677
     #           https://github.com/Miserlou/Zappa/issues/683
     #           https://github.com/Miserlou/Zappa/issues/696
@@ -77,12 +69,12 @@ def create_wsgi_request(
             body = base64.b64decode(encoded_body)
         else:
             body = event_info["body"]
-            if isinstance(body, six.string_types):
+            if isinstance(body, str):
                 body = body.encode("utf-8")
 
     else:
         body = event_info["body"]
-        if isinstance(body, six.string_types):
+        if isinstance(body, str):
             body = body.encode("utf-8")
 
     # Make header names canonical, e.g. content-type => Content-Type
@@ -100,7 +92,8 @@ def create_wsgi_request(
     if "," in x_forwarded_for:
         # The last one is the cloudfront proxy ip. The second to last is the real client ip.
         # Everything else is user supplied and untrustworthy.
-        remote_addr = x_forwarded_for.split(", ")[-2]
+        addresses = [addr.strip() for addr in x_forwarded_for.split(",")]
+        remote_addr = addresses[-2]
     else:
         remote_addr = x_forwarded_for or "127.0.0.1"
 
@@ -122,13 +115,24 @@ def create_wsgi_request(
         "wsgi.run_once": False,
     }
 
+    # Systems calling the Lambda (other than API Gateway) may not provide the field requestContext
+    # Extract remote_user, authorizer if Authorizer is enabled
+    remote_user = None
+    if "requestContext" in event_info:
+        authorizer = event_info["requestContext"].get("authorizer", None)
+        if authorizer:
+            remote_user = authorizer.get("principalId")
+            environ["API_GATEWAY_AUTHORIZER"] = authorizer
+        elif event_info["requestContext"].get("identity"):
+            remote_user = event_info["requestContext"]["identity"].get("userArn")
+
     # Input processing
     if method in ["POST", "PUT", "PATCH", "DELETE"]:
         if "Content-Type" in headers:
             environ["CONTENT_TYPE"] = headers["Content-Type"]
 
         # This must be Bytes or None
-        environ["wsgi.input"] = six.BytesIO(body)
+        environ["wsgi.input"] = BytesIO(body)
         if body:
             environ["CONTENT_LENGTH"] = str(len(body))
         else:
@@ -148,9 +152,6 @@ def create_wsgi_request(
     if remote_user:
         environ["REMOTE_USER"] = remote_user
 
-    if event_info["requestContext"].get("authorizer"):
-        environ["API_GATEWAY_AUTHORIZER"] = event_info["requestContext"]["authorizer"]
-
     return environ
 
 
@@ -164,24 +165,15 @@ def common_log(environ, response, response_time=None):
     logger = logging.getLogger()
 
     if response_time:
-        formatter = ApacheFormatter(with_response_time=True)
-        try:
-            log_entry = formatter(
-                response.status_code,
-                environ,
-                len(response.content),
-                rt_us=response_time,
-            )
-        except TypeError:
-            # Upstream introduced a very annoying breaking change on the rt_ms/rt_us kwarg.
-            log_entry = formatter(
-                response.status_code,
-                environ,
-                len(response.content),
-                rt_ms=response_time,
-            )
+        formatter = ApacheNCSAFormatter(with_response_time=True)
+        log_entry = formatter(
+            response.status_code,
+            environ,
+            len(response.content),
+            rt_us=response_time,
+        )
     else:
-        formatter = ApacheFormatter(with_response_time=False)
+        formatter = ApacheNCSAFormatter(with_response_time=False)
         log_entry = formatter(response.status_code, environ, len(response.content))
 
     logger.info(log_entry)
